#version 410

in vec4 clipSpace;
in vec4 worldPosition;
in vec2 textureCoords;

layout(location = 0) out vec4 fragmentColor;

uniform sampler2D reflectionTexture;
uniform sampler2D refractionTexture;
uniform sampler2D dudvMap;
uniform sampler2D normalMap;
uniform vec3 lightDirection;
uniform vec3 lightColor;
uniform vec3 cameraPosition;
uniform float moveFactor;

const float waveStrength = 0.04;
const float shineDamper = 20.0;
const float reflectivity = 0.5;

struct SpotLight {
    //Positions- und Beleuchtungs-Angaben
    vec3 position;  
    vec3 direction;
    vec3 color;

    //Gren der inneren und usseren Beleuchtungskreise
    float innerCone;  //cos(rad(wert))
    float outerCone; //cos(rad(wert))

    //Faktoren fr Lichtabschwchung auf Distanz
    float constant;
    float linear;
    float quadratic;
}; 

uniform vec3 spotlightPosition;
uniform vec3 spotlightDirection;
uniform vec3 spotlightColor;
uniform int night;

SpotLight spotLight = SpotLight(spotlightPosition, //position
                                spotlightDirection, //direction
                                spotlightColor,  //color
                                0.976, 0.953, //innerCone = cos(rad(12.5)), outerCone = cos(rad(17.5))
                                1.0, 0.004, 0.000007); //Lichtabschwaechung

void main(void) {
    
    vec3 myLightColor = lightColor;
    if(night == 1)
    {
        myLightColor = vec3(0.0, 0.0, 0.0);
    }

    vec2 ndc = (clipSpace.xy / clipSpace.w) / 2.0 + 0.5;
    vec2 refractTexCoords = vec2(ndc.x, ndc.y);
    vec2 reflectTexCoords = vec2(ndc.x, -ndc.y);

    vec2 distortedTexCoords = texture(dudvMap, vec2(textureCoords.x + moveFactor, textureCoords.y)).rg * 0.1;
    distortedTexCoords = textureCoords + vec2(distortedTexCoords.x, distortedTexCoords.y + moveFactor);
    vec2 totalDistortion = (texture(dudvMap, distortedTexCoords).rg * 2.0 - 1.0) * waveStrength;
    
    refractTexCoords += totalDistortion;
    refractTexCoords = clamp(refractTexCoords, 0.001, 0.999);
    
    reflectTexCoords += totalDistortion;
    reflectTexCoords.x = clamp(reflectTexCoords.x, 0.001, 0.999);
    reflectTexCoords.y = clamp(reflectTexCoords.y, -0.999, -0.001);
    
    vec4 reflectColour = texture(reflectionTexture, reflectTexCoords);
    vec4 refractColour = texture(refractionTexture, refractTexCoords);
    
    vec4 normalMapColour = texture(normalMap, distortedTexCoords);
    vec3 normal = vec3(normalMapColour.r * 2.0 - 1.0, normalMapColour.b*3.0, normalMapColour.g * 2.0 - 1.0);
    normal = normalize(normal);

    vec3 lightVector = normalize(-lightDirection);
    float diffuse = clamp(dot(lightVector, normal), 0.0, 1.0);

    vec3 viewVector = normalize(cameraPosition - worldPosition.xyz);
    vec3 reflectionVector = normalize(dot(2.0 * normal, lightVector) * normal - lightVector);

    vec3 reflectedLight = reflect(normalize(-lightVector), normal);
    float specular = max(dot(reflectedLight, viewVector), 0.0);
    specular = pow(specular, shineDamper);
    vec3 specularHighlights = myLightColor * specular * reflectivity;

    float refractiveFactor = dot(viewVector, normal);
    refractiveFactor = pow(refractiveFactor, 0.5);
    refractiveFactor = clamp(refractiveFactor - 0.5, 0.0, 1.0);

    fragmentColor = mix(reflectColour, refractColour, refractiveFactor);
    fragmentColor = mix(fragmentColor, vec4(0.0, 0.3, 0.5, 1.0), 0.2) + vec4(specularHighlights, 0.0);

    if(night == 1)
    {
        //Spotlight berechnungen
        vec3 lightDirection = normalize(spotLight.position - worldPosition.xyz);
        float theta = dot(lightDirection, normalize(-spotLight.direction)); //cos(rad(theta))
        if(theta > spotLight.outerCone) //Fragment innerhalb des Spotlight-Kreises
        {       
        float NdotLSpot = clamp(dot(normal, lightDirection), 0.0, 1.0);

        // Dmpfung berechnen (Entfernung)
        float distance    = length(spotLight.position - worldPosition.xyz);
        float attenuation = 1.0f / (spotLight.constant + spotLight.linear * distance + spotLight.quadratic * (distance * distance));
      
        //Dmpfung berechnen (Intensitaet outerCone)
        float epsilon = (spotLight.innerCone - spotLight.outerCone);
        float intensity = clamp((theta - spotLight.outerCone) / epsilon, 0.0, 1.0);

        fragmentColor = fragmentColor + vec4(spotLight.color * NdotLSpot, 1.0) * intensity * attenuation;
        }
    }

    fragmentColor = clamp(fragmentColor, 0.0, 1.0);
}